﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HEAL.Attic;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment.Algorithms {
  [Item("Population Context", "A context for population-based algorithms.")]
  [StorableType("328AF409-9ECE-4E59-A842-ABBD98CBE386")]
  public abstract class PopulationContext<TSolutionScope> : StochasticContext
    where TSolutionScope: class, IScope {

    public IEnumerable<TSolutionScope> Population {
      get { return Scope.SubScopes.OfType<TSolutionScope>(); }
    }
    public void AddToPopulation(TSolutionScope solScope) {
      Scope.SubScopes.Add(solScope);
    }
    public void ReplaceAtPopulation(int index, TSolutionScope solScope) {
      Scope.SubScopes[index] = solScope;
    }
    public void ReplacePopulation(IEnumerable<TSolutionScope> replacement) {
      Scope.SubScopes.Replace(replacement);
    }
    public TSolutionScope AtPopulation(int index) {
      return Scope.SubScopes[index] as TSolutionScope;
    }
    public TSolutionScope AtRandomPopulation() {
      return Scope.SubScopes[Random.Next(PopulationCount)] as TSolutionScope;
    }
    public int PopulationCount {
      get { return Scope.SubScopes.Count; }
    }


    [StorableConstructor]
    protected PopulationContext(StorableConstructorFlag _) : base(_) { }
    protected PopulationContext(PopulationContext<TSolutionScope> original, Cloner cloner)
      : base(original, cloner) {
    }
    protected PopulationContext() : base() { }
    protected PopulationContext(string name) : base(name) { }
    protected PopulationContext(string name, ParameterCollection parameters) : base(name, parameters) { }
    protected PopulationContext(string name, string description) : base(name, description) { }
    protected PopulationContext(string name, string description, ParameterCollection parameters) : base(name, description, parameters) { }
  }
}
